import json
import multiprocessing
import threading
import time
import traceback

from run.exception.local_exceptions import NumberProcessException
from run.redis_connector import RedisPool


# rpop from redis given key as list name
# return rpop result
def single_start(key, pid, func):
    """
    read list-key from redis,
    pop list element,
    call function with popped element as param
    :param key: model name defined in tool and model
    :param pid: process id
    :param func: call model function
    """
    print('process start: 【' + str(pid) + '】')
    pool = RedisPool()
    conn = pool.connector()

    while True:
        val = conn.brpop(key)
        print(pid, val[1])

        # do ack
        # do persistance work

        try:
            val_json = json.loads(val[1])
            func(val_json)
            # do exception work
        except:
            traceback.print_exc()
            # record failed vals
            err_json = {"val": val[1], "timestamp": int(round(time.time() * 1000))}
            conn.lpush('failed_' + key, json.dumps(err_json))


def start(key: str, func, np: int = 5):
    """
    call np number of processes,
    do func work, func params read from redis
    :param key: model name defined in tool and model
    :param func: call model function
    :param np: number of processes, default 5
    """

    if np < 1:
        raise NumberProcessException(np)
        return

    for i in range(np):
        p = multiprocessing.Process(target=single_start, kwargs={"key": key, "pid": i, "func": func})
        p.start()

        time.sleep(0.5)



import setuptools

setuptools.setup(
    name="zzha529_test",
    version="1.12",
    author="zzha529",
    description="personal redis queue consumer",
    long_description="personal redis queue consumer",
    packages=setuptools.find_packages(),
    install_requires=["redis==4.1.0"],
    python_requires=">3",
)




import json
import redis
import random


def singleton(cls, *args, **kw):
    instances = {}

    def _singleton():
        if cls not in instances:
            instances[cls] = cls(*args, **kw)
        return instances[cls]

    return _singleton


@singleton
class RedisPool(object):
    def __init__(self):
        with open(r'./redis_config.json', 'r', encoding='utf8') as f_load:
            redis_json = json.load(f_load)
        self.port = redis_json['port']
        self.host = redis_json['host']
        self.db = redis_json['db']
        self.random = random.random()
        self.pool = redis.ConnectionPool(host=self.host, port=self.port, db=self.db, decode_responses=True)

    def connector(self):
        return redis.Redis(connection_pool=self.pool, decode_responses=True)
